export declare const int32: Int32Array<ArrayBuffer>;
export declare const float32: Float32Array<ArrayBuffer>;
export declare const float64: Float64Array<ArrayBuffer>;
export declare const isLittleEndian: boolean;
