export var BitWidth;
(function (BitWidth) {
    BitWidth[BitWidth["WIDTH8"] = 0] = "WIDTH8";
    BitWidth[BitWidth["WIDTH16"] = 1] = "WIDTH16";
    BitWidth[BitWidth["WIDTH32"] = 2] = "WIDTH32";
    BitWidth[BitWidth["WIDTH64"] = 3] = "WIDTH64";
})(BitWidth || (BitWidth = {}));
